local garm = {}




--[[--
Copies a table
table copy_of_mytable = garm.copy(table mytable)
--]]--
garm.copy = function(obj, seen)
  if type(obj) ~= 'table' then return obj end
  if seen and seen[obj] then return seen[obj] end
  local s = seen or {}
  local res = setmetatable({}, getmetatable(obj))
  s[obj] = res
  for k, v in pairs(obj) do res[garm.copy(k, s)] = garm.copy(v, s) end
  return res
end




--[[--
Rotates an image
num x, num y = garm.rotateImage(image image, num angle)
Example:
local relx, rely = garm.rotateImage(myImage, math.pi*0.5)
love.graphics.draw(image, totalx + relx, totaly + rely, math.pi*0.5)
--]]--
function garm.rotateImage(image, ang)
  ang = ang - math.pi*0.75 --upper left corner of the image (full revolution: 2*math.pi)
  local midx = image:getWidth()/2 --coordinates of the middle of the image
  local midy = image:getHeight()/2
  local dist = math.sqrt(midx^2 + midy^2) --radius of the circle
  local x = midx + math.cos(ang)*dist
  local y = midy + math.sin(ang)*dist
  return x, y
end




--[[--
Creates an arrow
num x1, num y1, num x2, num y2, num distance = garm.arrow(num angle, num scale, num middle_angle)
Example:
local x1, y1, x2, y2 = garm.arrow(math.pi/2, 10, 0.5)
love.graphics.line(x1, y1, 0, 0, x2, y2)
--]]--
function garm.arrow(ang, scale, mang)
	scale = scale or 1
	mang = mang or math.pi/3
	mang = mang/2
	local x1 = math.cos(ang + math.pi - mang)*scale
	local x2 = math.cos(ang + math.pi + mang)*scale
	local y1 = math.sin(ang + math.pi - mang)*scale
	local y2 = math.sin(ang + math.pi + mang)*scale
	local dist = math.cos(mang)*scale
	return x1, y1, x2, y2, dist
end




--[[--
Converts coordinates
num x, num y = garm.karth(num angle, num distance)
num angle, num distance = garm.polar(num x, num y)
--]]--
function garm.karth(ang, dist)
	dist = dist or 1
	local x = math.cos(ang)*dist
	local y = math.sin(ang)*dist
	return x, y
end

function garm.polar(x, y)
	local ang = math.atan2(y, x)
	local dist = (x^2+y^2)^0.5
	return ang, dist
end




--[[--
Converts HSV/HSL colors to RGB colors (providing a is optional)
num r, num g, num b, num a = garm.HSV(num h, num s, num v, num a)
num r, num g, num b, num a = garm.HSL(num h, num s, num l, num a)
--]]--
function garm.HSV(h, s, v, a)
	if s <= 0 then return v,v,v,a end
	h, s, v = h/256*6, s/255, v/255
	local c = v*s
	local x = (1-math.abs((h%2)-1))*c
	local m,r,g,b = (v-c), 0,0,0
	if h < 1     then r,g,b = c,x,0
	elseif h < 2 then r,g,b = x,c,0
	elseif h < 3 then r,g,b = 0,c,x
	elseif h < 4 then r,g,b = 0,x,c
	elseif h < 5 then r,g,b = x,0,c
	else              r,g,b = c,0,x
	end return (r+m)*255,(g+m)*255,(b+m)*255,a
end

function garm.HSL(h, s, l, a)
	if s<=0 then return l,l,l,a end
	h, s, l = h/256*6, s/255, l/255
	local c = (1-math.abs(2*l-1))*s
	local x = (1-math.abs(h%2-1))*c
	local m,r,g,b = (l-.5*c), 0,0,0
	if h < 1     then r,g,b = c,x,0
	elseif h < 2 then r,g,b = x,c,0
	elseif h < 3 then r,g,b = 0,c,x
	elseif h < 4 then r,g,b = 0,x,c
	elseif h < 5 then r,g,b = x,0,c
	else              r,g,b = c,0,x
	end return (r+m)*255,(g+m)*255,(b+m)*255,a
end




return garm